==> typescript-ioc-3.0.1.tgz: package/src/container/container.ts <==
import { InjectorHandler } from './injection-handler';
import { Scope, ObjectFactory, Snapshot, BuildContext } from '../model';
import { IoCBindConfig } from './container-binding-config';

/**
 * Internal implementation of IoC Container.
 */
export class IoCContainer {
    public static bind(source: Function): IoCBindConfig {
        InjectorHandler.checkType(source);
        const baseSource = InjectorHandler.getConstructorFromType(source);
        let config: IoCBindConfig = IoCContainer.bindings.get(baseSource);
        if (!config) {
            config = new IoCBindConfig(baseSource, IoCContainer.get);
            config
                .to(source as FunctionConstructor);
            IoCContainer.bindings.set(baseSource, config);
        }
        return config;
    }

    public static get(source: Function, context: BuildContext) {
        const config: IoCBindConfig = IoCContainer.bind(source);
        if (!config.iocFactory) {
            config.to(config.source as FunctionConstructor);
        }
        return config.getInstance(context);
    }

    public static getType(source: Function): Function {
        InjectorHandler.checkType(source);
        const baseSource = InjectorHandler.getConstructorFromType(source);
        const config: IoCBindConfig = IoCContainer.bindings.get(baseSource);
        if (!config) {
            throw new TypeError(`The type ${source.name} hasn't been registered with the IOC Container`);
        }
        return config.targetSource || config.source;
    }

    public static injectProperty(target: Function, key: string, propertyType: Function) {
        InjectorHandler.injectProperty(target, key, propertyType, IoCContainer.get);
    }

    /**
     * Store the state for a specified binding.  Can then be restored later.   Useful for testing.
     * @param source The dependency type
     */
    public static snapshot(source: Function): Snapshot {
        const config = IoCContainer.bind(source);
        IoCContainer.snapshots.set(source, {
            factory: config.iocFactory,
            scope: config.iocScope,
            withParams: config.paramTypes
        });
        return {
            restore: () => IoCContainer.restore(source)
        };
    }

    /**
     * Restores the state for a specified binding that was previously captured by snapshot.
     * @param source The dependency type
     */
    private static restore(source: Function): void {
        const config = IoCContainer.bind(source);
        const configSnapshot = IoCContainer.snapshots.get(source);
        config
            .factory(configSnapshot.factory)
            .scope(configSnapshot.scope)
            .withParams(configSnapshot.withParams);
    }

    /**
     * Internal storage for snapshots
     * @type {providers: Map<Function, Provider>; scopes: Map<Function, Scope>}
     */
    private static snapshots: Map<Function, ConfigSnapshot> = new Map();

    private static bindings: Map<FunctionConstructor, IoCBindConfig> = new Map<FunctionConstructor, IoCBindConfig>();
}

interface ConfigSnapshot {
    factory: ObjectFactory;
    scope: Scope;
    withParams: Array<any>;
}

==> typescript-ioc-3.1.0.tgz: package/src/container/container.ts <==
import { InjectorHandler } from './injection-handler';
import { Scope, ObjectFactory, Snapshot, BuildContext } from '../model';
import { IoCBindConfig, IoCBindValueConfig, PropertyPath } from './container-binding-config';

/**
 * Internal implementation of IoC Container.
 */
export class IoCContainer {
    public static bind(source: Function): IoCBindConfig {
        InjectorHandler.checkType(source);
        const baseSource = InjectorHandler.getConstructorFromType(source);
        let config: IoCBindConfig = IoCContainer.bindings.get(baseSource);
        if (!config) {
            config = new IoCBindConfig(baseSource, IoCContainer.get, IoCContainer.getValue);
            config
                .to(source as FunctionConstructor);
            IoCContainer.bindings.set(baseSource, config);
        }
        return config;
    }

    public static bindName(name: string): IoCBindValueConfig {
        InjectorHandler.checkName(name);
        const property = PropertyPath.parse(name);
        let config = IoCContainer.values.get(property.name);
        if (!config) {
            config = new IoCBindValueConfig(property.name);
            IoCContainer.values.set(property.name, config);
        }
        config.path = property.path;
        return config;
    }

    public static get(source: Function, context: BuildContext) {
        const config: IoCBindConfig = IoCContainer.bind(source);
        if (!config.iocFactory) {
            config.to(config.source as FunctionConstructor);
        }
        return config.getInstance(context);
    }

    public static getValue(name: string) {
        const config: IoCBindValueConfig = IoCContainer.bindName(name);
        return config.getValue();
    }

    public static getType(source: Function): Function {
        InjectorHandler.checkType(source);
        const baseSource = InjectorHandler.getConstructorFromType(source);
        const config: IoCBindConfig = IoCContainer.bindings.get(baseSource);
        if (!config) {
            throw new TypeError(`The type ${source.name} hasn't been registered with the IOC Container`);
        }
        return config.targetSource || config.source;
    }

    public static injectProperty(target: Function, key: string, propertyType: Function) {
        InjectorHandler.injectProperty(target, key, propertyType, IoCContainer.get);
    }

    public static injectValueProperty(target: Function, key: string, name: string) {
        InjectorHandler.injectValueProperty(target, key, name, IoCContainer.getValue);
    }

    /**
     * Store the state for a specified binding.  Can then be restored later.   Useful for testing.
     * @param source The dependency type
     */
    public static snapshot(source: Function): Snapshot {
        const config = IoCContainer.bind(source);
        IoCContainer.snapshots.set(source, {
            factory: config.iocFactory,
            scope: config.iocScope,
            withParams: config.paramTypes
        });
        return {
            restore: () => IoCContainer.restore(source)
        };
    }

    /**
     * Restores the state for a specified binding that was previously captured by snapshot.
     * @param source The dependency type
     */
    private static restore(source: Function): void {
        const config = IoCContainer.bind(source);
        const configSnapshot = IoCContainer.snapshots.get(source);
        config
            .factory(configSnapshot.factory)
            .scope(configSnapshot.scope)
            .withParams(configSnapshot.withParams);
    }

    /**
     * Internal storage for snapshots
     * @type {providers: Map<Function, Provider>; scopes: Map<Function, Scope>}
     */
    private static snapshots: Map<Function, ConfigSnapshot> = new Map();

    private static bindings: Map<FunctionConstructor, IoCBindConfig> = new Map();
    private static values: Map<string, IoCBindValueConfig> = new Map();
}

interface ConfigSnapshot {
    factory: ObjectFactory;
    scope: Scope;
    withParams: Array<any>;
}
